var lingo = require('lingo')
  , fs = require('fs')
  , path = require('path')
  , _ = require('underscore');

// To be exported ...
var fixtures = module.exports = {};
var collections = module.exports.collections = [];

var get = function (obj, path) {
  var segments = path.split('.')
    , cursor = obj
    , segment, i;
  
  for (i = 0; i < segments.length - 1; ++i) {
    segment = segments[i];
    cursor = cursor[segment] = cursor[segment] || {};
  }
  
  return cursor[segments[i]];
};

/**
 * Loads fixtures from the given path.
 * @param {string} fixture_path The path to the fixtures
 */
var load = module.exports.load = function(fixture_path) {
  var files = fs.readdirSync(fixture_path||"./fixtures");
  _.each(files, function(file) {
    var collectionName = path.basename(file, ".json");
    collections.push(collectionName);
    fixtures[collectionName] = JSON.parse(fs.readFileSync(path.join(fixture_path||"./fixtures",file)));
  });  
};

/**
 * Persists the fixtures to the database, clears any and all collections present in the fixtures.
 * @param {object} db Database connection
 * @param {function} cb Callback function once completed
 */
var save = module.exports.save = function(db, cb) {
  var totalCollectionNr = 0;
  var currentCollectionCounter = 0;
  var totalRecordCounter = 0;
  var currentRecordCounter = 0;
  
  db.open(function(err, _db) {

    totalCollectionNr = collections.length;
    // Walk through the collections
    _.each(collections, function(collectionName) {

      currentCollectionCounter++;

      db.collection(collectionName, function(err, collection) { 

        // Trash the collection in the database
        collection.remove({}, function(err, result){
          
          totalRecordCounter += _.keys(fixtures[collectionName]).length;
          
          // Walk through a specific fixture's records
          _.each(_.keys(fixtures[collectionName]), function(recordId) {

            collection.insert(fixtures[collectionName][recordId], function(e, docs) {
              currentRecordCounter++;

              // Update the fixtures with the id
              fixtures[collectionName][recordId]['_id'] = docs[0]['_id'];
              
              // Are we done?
              if ( currentCollectionCounter === totalCollectionNr && currentRecordCounter === totalRecordCounter) {
                populate(db, cb);
              }
              
            });
      
          });        
        
        });

      });
    });
  });
};

/** @ignore */
var populate = function(db, cb) {
  var propertyPlural;

  var totalCollectionNr = 0;
  var currentCollectionCounter = 0;
  var totalRecordCounter = 0;
  var currentRecordCounter = 0;
  
  var guide;

  totalCollectionNr = collections.length;

  // Walk through the collections
  _.each(collections, function(collectionName) {

    currentCollectionCounter++;
  
    db.collection(collectionName, function(err, collection) { 

      totalRecordCounter += _.keys(fixtures[collectionName]).length;
  
      // Walk through a specific fixture's records
      _.each(_.keys(fixtures[collectionName]), function(recordId) {

        currentRecordCounter++;
        
        // Walk through the keys of a record
        _.each(_.keys(fixtures[collectionName][recordId]), function(propertyName) {

          // If the key of a record is a singular of a fixture, do our magic.
          // TODO: It would probably be faster to singularize the collections and look up, but
          // that doesn't always produce correct results.
          propertyPlural = fixtures[lingo.en.pluralize(propertyName)];
          if (propertyPlural) {
            
            if(typeof(fixtures[collectionName][recordId][propertyName]) === "string") {
              fixtures[collectionName][recordId][propertyName] = get(propertyPlural, fixtures[collectionName][recordId][propertyName]);
              
            } else {
              // We expect a path and 'key' in the object 
              guide = fixtures[collectionName][recordId][propertyName];  
              fixtures[collectionName][recordId][guide.key] = get(propertyPlural, guide.path);
                            
              // Remove the "guide"
              if (guide.as !== propertyName) {
                delete fixtures[collectionName][recordId][propertyName];
              }
            }
            
            //fixtures[collectionName][recordId][propertyName] = get(propertyPlural, fixtures[collectionName][recordId][propertyName]);
            collection.update({'_id': fixtures[collectionName][recordId]._id}, fixtures[collectionName][recordId], true, function(e, docs){
              if (e) {
                throw e;
              }
              if ( currentCollectionCounter === totalCollectionNr && currentRecordCounter === totalRecordCounter) {
                cb();
              }
              
            });
        
          }

        });

      });
    
    });
  
  });


};

